package eu.semiramis.policymodel.policy;

import java.util.Iterator;

import javax.xml.parsers.ParserConfigurationException;

import org.opensaml.xacml.ctx.RequestType;
import org.opensaml.xacml.policy.ApplyType;
import org.opensaml.xacml.policy.AttributeValueType;
import org.opensaml.xacml.policy.ObligationType;
import org.opensaml.xacml.policy.ObligationsType;
import org.opensaml.xacml.policy.PolicyType;
import org.opensaml.xml.ConfigurationException;

import eu.semiramis.policymodel.utils.PolicyModelXACMLPolicyUtils;

public class AttributeReleasePolicy extends PolicyModelXACMLPolicyUtils implements Policy {

	public final static String ATTRRELEASE_SUBJECT = "urn:semiramis:names:accesscontrol:attributerelease:subject:requester";
	//public final static String ATTRRELEASE_RESOURCE_PROVIDER = "urn:semiramis:names:accesscontrol:attributerelease:resource:provider";
	//ATTRIBUTE_ID below references to the attribute as a "attrProvider;attrOwner;attrName"
	public final static String ATTRRELEASE_RESOURCE_ATTRIBUTE_ID = "urn:semiramis:names:accesscontrol:attributerelease:resource:attribute-id";
	public final static String ATTRRELEASE_RESOURCE_ATTRIBUTE_PROVIDER = "urn:semiramis:names:accesscontrol:attributerelease:resource:attribute-provider";
	public final static String ATTRRELEASE_RESOURCE_ATTRIBUTE_OWNER = "urn:semiramis:names:accesscontrol:attributerelease:resource:attribute-owner";
	public final static String ATTRRELEASE_RESOURCE_ATTRIBUTE_NAME = "urn:semiramis:names:accesscontrol:attributerelease:resource:attribute-name";
	public final static String ATTRRELEASE_OBLIGATION_ID_CONSENT = "urn:semiramis:names:accesscontrol:attributerelease:obligation:user-consent";
	public final static String ATTRRELEASE_OBLIGATION_ID_NOTIFICATION = "urn:semiramis:names:accesscontrol:attributerelease:obligation:notify-user";
	
	private String requester; //subject - originator of the request
	private String attrProvider = ""; //atribute Provider name, URL, etc; resource 1
	private String attributeOwner = ""; //data subject ID; resource 2
	private String attributeName = ""; //resource 3
	private String policyId = "";
	private String description = "";
	private boolean consentRequired; //optional obligation
	private boolean notificationRequired; //optional obligation
	private String combiningAlgorithm;
	
	/*******  Constructors  *******/
	public AttributeReleasePolicy(String policyId, String description, String attrProvider, String attributeOwner, String attributeName, String requester, boolean consentRequired, boolean notificationRequired, String combiningAlgorithm) {
		this.attributeName = attributeName;
		this.attributeOwner = attributeOwner;
		this.attrProvider = attrProvider;
		this.policyId = policyId;
		this.description = description;
		this.requester = requester;
		this.consentRequired = consentRequired;
		this.notificationRequired = notificationRequired;
		this.combiningAlgorithm = combiningAlgorithm;
	}
	
	

	//creates a Policy Model AttributeReleasePolicy from XACML PolicyType,
	//equivalent to the "fromXACMLPolicy()" method
	public AttributeReleasePolicy(PolicyType xacmlPolicy) {
		try {
			init();
		} catch (ConfigurationException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
		PolicyType xacmlPol = unMarshallXACMLPolicy(xacmlPolicy);
		this.requester = xacmlPol.getTarget().getSubjects().getSubjects().get(0).getSubjectMatches().
			get(0).getAttributeValue().getValue();
		
		for (int i=0; i<xacmlPol.getTarget().getResources().getResources().get(0).getResourceMatches().size(); i++){
		   String attribute_id	= xacmlPol.getTarget().getResources().getResources().get(0).getResourceMatches().
					get(i).getResourceAttributeDesignator().getAttributeId();
		   if (attribute_id.equals(ATTRRELEASE_RESOURCE_ATTRIBUTE_PROVIDER)) this.attrProvider = xacmlPol.getTarget().getResources().
		   		getResources().get(0).getResourceMatches().get(i).getAttributeValue().getValue();
		   if (attribute_id.equals(ATTRRELEASE_RESOURCE_ATTRIBUTE_OWNER)) this.attributeOwner = xacmlPol.getTarget().getResources().
	   			getResources().get(0).getResourceMatches().get(i).getAttributeValue().getValue();
		   if (attribute_id.equals(ATTRRELEASE_RESOURCE_ATTRIBUTE_NAME)) this.attributeName = xacmlPol.getTarget().getResources().
  				getResources().get(0).getResourceMatches().get(i).getAttributeValue().getValue();
		   	  
		}
	   	this.policyId = xacmlPol.getPolicyId();
	   	this.description = xacmlPol.getDescription().getValue();
	   
	   	/*The commented code below is to obtain the attrProvider, attrOwner, and attrName from the "condition" part of the policy
	   	ApplyType apply = (ApplyType)xacmlPol.getRules().get(0).getCondition().getExpression();
	   	AttributeValueType conditionValue = (AttributeValueType) apply.getExpressions().get(0);
	   	String combinedResource = conditionValue.getValue();
	   	this.attrProvider = combinedResource.substring(0, combinedResource.indexOf(";"));
		this.attributeOwner = combinedResource.substring(combinedResource.indexOf(";") + 1, combinedResource.lastIndexOf(";"));
		this.attributeName = combinedResource.substring(combinedResource.lastIndexOf(";") + 1, combinedResource.length());
	   	//ApplyType applyWrapResAttrDesignator = (ApplyType) apply.getExpressions().get(0);
	   	*/
	   		
		if (xacmlPol.getRuleCombiningAlgoId().equals("urn:oasis:names:tc:xacml:1.0:rule-combining-algorithm:permit-overrides")){
			this.combiningAlgorithm = "permit";
		} else {
			this.combiningAlgorithm = "deny";
		}
		
	   	
		ObligationsType obligations = xacmlPol.getObligations();
		this.consentRequired = false;
		this.notificationRequired = false;
	   	if (obligations != null){
	   		Iterator itr = obligations.getObligations().iterator(); 
	   		while(itr.hasNext()) {	   		
	   			ObligationType ob = (ObligationType) itr.next();
	   			String obligation_id = ob.getObligationId();
	   			if (obligation_id.equals(AttributeReleasePolicy.ATTRRELEASE_OBLIGATION_ID_CONSENT)) this.consentRequired = true;
	   			if (obligation_id.equals(AttributeReleasePolicy.ATTRRELEASE_OBLIGATION_ID_NOTIFICATION)) this.notificationRequired = true;	   				    	   		
	   		}
	   	}
		
	}
	
	/*********  Methods  *********/
	@Override
	public PolicyType toXACMLPolicy() {
		try {
			init();
		} catch (ConfigurationException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
		return buildXACMLPolicy(this);
	}
	
	/********Setters Getters********/
	
	public String getCombiningAlgorithm() {
		return combiningAlgorithm;
	}

	public void setCombiningAlgorithm(String combiningAlgorithm) {
		this.combiningAlgorithm = combiningAlgorithm;
	}
	
	public void setAttributeName(String attributeName) {
		this.attributeName = attributeName;
	}
	public String getAttributeName() {
		return attributeName;
	}
	
	public void setAttrProvider(String attrProvider) {
		this.attrProvider = attrProvider;
	}
	public String getAttrProvider() {
		return attrProvider;
	}
	
	public void setAttributeOwner(String attributeOwner) {
		this.attributeOwner = attributeOwner;
	}
	public String getAttributeOwner() {
		return attributeOwner;
	}
	
	public void setRequester(String requester) {
		this.requester = requester;
	}
	public String getRequester() {
		return requester;
	}
	
	public void setConsentRequired(boolean consentRequired) {
		this.consentRequired = consentRequired;
	}
	public boolean getConsentRequired() {
		return consentRequired;
	}
	
	public void setNotificationRequired(boolean notificationRequired) {
		this.notificationRequired = notificationRequired;
	}
	public boolean getNotificationRequired() {
		return notificationRequired;
	}
	
	@Override
	public String getPolicyId() {
		return policyId;
	}
	
	@Override
	public void setPolicyId(String policyId) {
		this.policyId = policyId;
	}
	
	@Override
	public String getDescription(){
		return description;
	}
	
	@Override
	public void setDescription(String description){
		this.description = description;
	}
}
