package eu.semiramis.policymodel.policy;

import javax.xml.parsers.ParserConfigurationException;

import org.opensaml.xacml.policy.ApplyType;
import org.opensaml.xacml.policy.AttributeValueType;
import org.opensaml.xacml.policy.PolicyType;
import org.opensaml.xacml.policy.impl.ApplyTypeImpl;
import org.opensaml.xml.ConfigurationException;

import eu.semiramis.policymodel.utils.PolicyModelXACMLPolicyUtils;

public class RoleBasedAccessPolicy extends PolicyModelXACMLPolicyUtils implements Policy {

	public static final String RBAC_SUBJECT_ID = "urn:semiramis:names:accesscontrol:rbac:subject:role-id";
	public static final String RBAC_ACTION_ID = "urn:semiramis:names:accesscontrol:rbac:action:functionality-id";
	public static final String RBAC_RESOURCE_ID	= "urn:semiramis:names:accesscontrol:rbac:resource:resource-id";
	
	private String requestedResource; //resource
	private String role; //subject
	private String policyId;
	private String description;
	private String functionalityId; //action
	
	/*******  Constructors  *******/
	public RoleBasedAccessPolicy(String policyId, String role, String requestedResource, String functionality) {
		this.role = role;
		this.policyId = policyId;
		this.requestedResource = requestedResource;
		this.functionalityId = functionality;
	}
	
	//creates a Policy Model RoleBasedAccessPolicy from XACML PolicyType,
	//equivalent to the "fromXACMLPolicy()" method
	public RoleBasedAccessPolicy(PolicyType xacmlPolicy) {
		try {
			init();
		} catch (ConfigurationException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
		PolicyType xacmlPol = unMarshallXACMLPolicy(xacmlPolicy);
		//this.attributeId = xacmlPol.getTarget().getResources().getResources().get(0).getResourceMatches().
		//get(0).getResourceAttributeDesignator().getAttributeId();
	   	this.policyId = xacmlPol.getPolicyId();
	   	this.description = xacmlPol.getDescription().getValue();
	   	this.requestedResource = xacmlPol.getTarget().getResources().getResources().get(0).getResourceMatches().
	   		get(0).getAttributeValue().getValue();
	   	this.functionalityId = xacmlPol.getRules().get(0).getTarget().getActions().getActions().get(0).
	   		getActionMatches().get(0).getAttributeValue().getValue();
	   	ApplyType apply = (ApplyType)xacmlPol.getRules().get(0).getCondition().getExpression();
	   	//ApplyTypeImpl applyWrapSubAttrDesignator = (ApplyTypeImpl) apply.getExpressions().get(0);
	   	AttributeValueType conditionValue = (AttributeValueType) apply.getExpressions().get(0);
	   	
	   	//AttributeValueType conditionValue = (AttributeValueType) applyWrapSubAttrDesignator.getExpressions().get(1);
	   	this.role = conditionValue.getValue();
		
	}
	
	/*********  Methods  *********/
	@Override
	public PolicyType toXACMLPolicy() {
		try {
			init();
		} catch (ConfigurationException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
		return buildXACMLPolicy(this);
	}
	
	/********Setters Getters********/
	
	public void setRole(String role) {
		this.role = role;
	}
	public String getRole() {
		return role;
	}
	
	public void setRequestedResource(String requestedResource) {
		this.requestedResource = requestedResource;
	}
	public String getRequestedResource() {
		return requestedResource;
	}
	
	public void setFunctionalityId(String functionalityId) {
		this.functionalityId = functionalityId;
	}
	public String getFunctionalityId() {
		return functionalityId;
	}
	
	@Override
	public String getPolicyId() {
		return policyId;
	}
	
	@Override
	public void setPolicyId(String policyId) {
		this.policyId = policyId;
	}
	
	@Override
	public String getDescription(){
		return description;
	}
	
	@Override
	public void setDescription(String description){
		this.description = description;
	}
}

