package eu.semiramis.policymodel.policy;

import javax.xml.parsers.ParserConfigurationException;

import org.opensaml.xacml.policy.ApplyType;
import org.opensaml.xacml.policy.AttributeValueType;
import org.opensaml.xacml.policy.ExpressionType;
import org.opensaml.xacml.policy.PolicyType;
import org.opensaml.xacml.policy.SubjectType;
import org.opensaml.xml.ConfigurationException;

import eu.semiramis.policyManager.util.opensaml.XACMLUtils;
import eu.semiramis.policymodel.utils.PolicyModelXACMLPolicyUtils;

public class ServiceToServicePolicy extends PolicyModelXACMLPolicyUtils implements Policy {

	//urns ServiceToService policies
	public static final String IATOIA_SUBJECT_ID = "urn:semiramis:names:accesscontrol:IAtoIA:subject:invoking-service-id";
	public static final String IATOIA_ACTION_ID = "urn:semiramis:names:accesscontrol:IAtoIA:action:functionality-id";
	public static final String IATOIA_RESOURCE_ID = "urn:semiramis:names:accesscontrol:IAtoIA:resource:invoked-service-id";
	
	private String originService; //subject
	private String functionality; //action
	private String targetService; //resource
	private String policyId;
	private String description;
	
	/*******  Constructors  *******/
	public ServiceToServicePolicy(){}
	public ServiceToServicePolicy(String policyId, String originService, String functionality, String targetService) {
		this.policyId = policyId;
		this.originService = originService;
		this.functionality = functionality;
		this.targetService = targetService;		
	}
	
	//creates a Policy Model ServiceToServicePolicy from XACML PolicyType,
	//equivalent to the "fromXACMLPolicy()" method
	public ServiceToServicePolicy(PolicyType xacmlPolicy) {
		try {
			init();
		} catch (ConfigurationException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
		PolicyType xacmlPol = unMarshallXACMLPolicy(xacmlPolicy);
		//this.attributeId = xacmlPol.getTarget().getResources().getResources().get(0).getResourceMatches().
		//get(0).getResourceAttributeDesignator().getAttributeId();
	   	this.policyId = xacmlPol.getPolicyId();
	   	this.description = xacmlPol.getDescription().getValue();
	   	this.targetService = xacmlPol.getTarget().getResources().getResources().get(0).getResourceMatches().
	   		get(0).getAttributeValue().getValue();
	   	this.functionality = xacmlPol.getRules().get(0).getTarget().getActions().getActions().get(0).
	   		getActionMatches().get(0).getAttributeValue().getValue();
	   	ApplyType apply = (ApplyType)xacmlPol.getRules().get(0).getCondition().getExpression();
	   	ApplyType applyWrapSubAttrDesignator = (ApplyType) apply.getExpressions().get(0);
	   	//AttributeValueType conditionValue = (AttributeValueType) applyWrapSubAttrDesignator.getExpressions().get(1);
	   	AttributeValueType conditionValue = (AttributeValueType) apply.getExpressions().get(1);
	   	this.originService = conditionValue.getValue();
		
	}
	
	/*********  Methods  *********/
	@Override
	public PolicyType toXACMLPolicy() {
		try {
			init();
		} catch (ConfigurationException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
		return buildXACMLPolicy(this);
	}
	
	/****** Setters Getters ******/	
	public void setOriginService(String originService) {
		this.originService = originService;
	}
	public String getOriginService() {
		return originService;
	}
	
	public void setFunctionality(String functionality) {
		this.functionality = functionality;
	}
	public String getFunctionality() {
		return functionality;
	}
	
	public void setTargetService(String targetService) {
		this.targetService = targetService;
	}
	public String getTargetService() {
		return targetService;
	}
	@Override
	public String getPolicyId() {
		return policyId;
	}
	@Override
	public void setPolicyId(String policyId) {
		this.policyId = policyId;
	}
	
	@Override
	public String getDescription(){
		return description;
	}
	
	@Override
	public void setDescription(String description){
		this.description = description;
	}

}
