package eu.semiramis.policymodel.request;

import javax.xml.parsers.ParserConfigurationException;
import org.opensaml.xacml.ctx.RequestType;
import org.opensaml.xml.ConfigurationException;
import eu.semiramis.policymodel.utils.PolicyModelXACMLRequestUtils;

public class AttributeReleaseRequest extends PolicyModelXACMLRequestUtils implements Request {

	public final static String ATTRRELEASE_SUBJECT = "urn:semiramis:names:accesscontrol:attributerelease:subject:requester";
	public final static String ATTRRELEASE_RESOURCE_ATTRIBUTE_ID = "urn:semiramis:names:accesscontrol:attributerelease:resource:attribute-id";
	public final static String ATTRRELEASE_RESOURCE_ATTRIBUTE_PROVIDER = "urn:semiramis:names:accesscontrol:attributerelease:resource:attribute-provider";
	public final static String ATTRRELEASE_RESOURCE_ATTRIBUTE_OWNER = "urn:semiramis:names:accesscontrol:attributerelease:resource:attribute-owner";
	public final static String ATTRRELEASE_RESOURCE_ATTRIBUTE_NAME = "urn:semiramis:names:accesscontrol:attributerelease:resource:attribute-name";
	public final static String ATTRRELEASE_OBLIGATION_ID_CONSENT = "urn:semiramis:names:accesscontrol:attributerelease:obligation:user-consent";
	public final static String ATTRRELEASE_OBLIGATION_ID_NOTIFICATION = "urn:semiramis:names:accesscontrol:attributerelease:obligation:notify-user";
	
	private String requester;
	private String attrProvider;
	private String attributeOwner;
	private String requestedAttributeName;
	
	
	/*******  Constructors  *******/
	public AttributeReleaseRequest(String requester, String attrProvider, String attributeOwner, String requestedAttributeName) {
		
		this.requester = requester;
		this.attrProvider = attrProvider;
		this.attributeOwner = attributeOwner;
		this.requestedAttributeName = requestedAttributeName;
		
	}
	
	//creates a Policy Model AttributeReleaseRequest from XAML RequestType,
	//equivalent to the "fromXACMLRequest()" method
	public AttributeReleaseRequest(RequestType xacmlRequest) {
		try {
			init();
		} catch (ConfigurationException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
		RequestType xacmlReq = unMarshallXACMLRequest(xacmlRequest);
		this.attrProvider = xacmlReq.getResources().get(0).getAttributes().get(0).getAttributeValues().get(0).getValue();
		this.attributeOwner = xacmlReq.getResources().get(0).getAttributes().get(1).getAttributeValues().get(0).getValue();
		this.requestedAttributeName = xacmlReq.getResources().get(0).getAttributes().get(2).getAttributeValues().get(0).getValue();
		
		/*String combinedResource = xacmlReq.getResources().get(0).
			getAttributes().get(0).getAttributeValues().get(0).getValue();
		this.attrProvider = combinedResource.substring(0, combinedResource.indexOf(";"));
		this.attributeOwner = combinedResource.substring(combinedResource.indexOf(";") + 1, combinedResource.lastIndexOf(";"));
		this.requestedAttributeName = combinedResource.substring(combinedResource.lastIndexOf(";") + 1, combinedResource.length());*/
		this.requester = xacmlReq.getSubjects().get(0).
				getAttributes().get(0).getAttributeValues().get(0).getValue();
	}
	
	/*********  Methods	 **********/	
	
	@Override
	public RequestType toXACMLRequest() {
		try {
			init();
		} catch (ConfigurationException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
		return buildXACMLRequest(this);
	}

	/********Setters Getters********/
	public void setRequestedAttributeName(String requestedAttributeName) {
		this.requestedAttributeName = requestedAttributeName;
	}
	public String getRequestedAttributeName() {
		return requestedAttributeName;
	}
	
	public void setAttrProvider(String attrProvider) {
		this.attrProvider = attrProvider;
	}
	public String getAttrProvider() {
		return attrProvider;
	}
	
	public void setAttributeOwner(String attributeOwner) {
		this.attributeOwner = attributeOwner;
	}
	public String getAttributeOwner() {
		return attributeOwner;
	}
	
	public void setRequester(String requester) {
		this.requester = requester;
	}
	public String getRequester() {
		return requester;
	}
	
}
