package eu.semiramis.policymodel.request;

import javax.xml.parsers.ParserConfigurationException;
import org.opensaml.xacml.ctx.RequestType;
import org.opensaml.xml.ConfigurationException;
import org.opensaml.xml.io.MarshallingException;

import eu.semiramis.policymodel.utils.PolicyModelXACMLRequestUtils;


public class ServiceToServiceRequest extends PolicyModelXACMLRequestUtils implements Request{
	
	//urns ServiceToService requests
	public static final String IATOIA_SUBJECT_ID = "urn:semiramis:names:accesscontrol:IAtoIA:subject:invoking-service-id";
	public static final String IATOIA_ACTION_ID = "urn:semiramis:names:accesscontrol:IAtoIA:action:functionality-id";
	public static final String IATOIA_RESOURCE_ID = "urn:semiramis:names:accesscontrol:IAtoIA:resource:invoked-service-id";
	
	private String originService; //subject
	private String functionality; //action
	private String targetService; //resource
		
	
	/*******  Constructors  *******/
	public ServiceToServiceRequest(RequestType xacmlRequest){
		try {
			this.init();
		} catch (ConfigurationException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
		RequestType xacmlReq = unMarshallXACMLRequest(xacmlRequest);
		this.targetService = xacmlReq.getResources().get(0).
	   		getAttributes().get(0).getAttributeValues().get(0).getValue();
	    this.originService = xacmlReq.getSubjects().get(0).
	   		getAttributes().get(0).getAttributeValues().get(0).getValue();
	    this.functionality = xacmlReq.getAction().getAttributes().
	    	get(0).getAttributeValues().get(0).getValue();
	}
	
	public ServiceToServiceRequest(String originServiceId, String functionalityId, String targetServiceId) 
		throws ConfigurationException, MarshallingException {
		
		this.originService = originServiceId;
		this.functionality = functionalityId;
		this.targetService = targetServiceId;		
	}
	
	/*********  Methods  *********/	
	@Override
	public RequestType toXACMLRequest() {
		try {
			this.init();
		} catch (ConfigurationException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
		return buildXACMLRequest(this);
	}
	
	/********Setters Getters********/
	
	public void setOriginService(String originService) {
		this.originService = originService;
	}
	public String getOriginService() {
		return originService;
	}
	
	public void setFunctionality(String functionality) {
		this.functionality = functionality;
	}
	public String getFunctionality() {
		return functionality;
	}
	
	public void setTargetServiceId(String targetService) {
		this.targetService = targetService;
	}
	public String getTargetService() {
		return targetService;
	}

}

