package eu.semiramis.policymodel.response;

import java.util.Iterator;

import javax.xml.parsers.ParserConfigurationException;

import org.opensaml.xacml.ctx.DecisionType;
import org.opensaml.xacml.ctx.ResponseType;
import org.opensaml.xacml.ctx.StatusType;
import org.opensaml.xacml.policy.ObligationType;
import org.opensaml.xacml.policy.ObligationsType;
import org.opensaml.xml.ConfigurationException;

import eu.semiramis.policymodel.policy.AttributeReleasePolicy;
import eu.semiramis.policymodel.utils.PolicyModelXACMLResponseUtils;

public class AttributeReleaseResponseHandler extends PolicyModelXACMLResponseUtils implements ResponseHandler{

	private boolean authorized;
	private String statusCode;
	private String statusMessage;
	private boolean consentRequired;
	private boolean notificationRequired;
	
	
	@Override
	public void handle(ResponseType response) {
		try {
			this.init();
		} catch (ConfigurationException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
		ResponseType xacmlResponse = unMarshallXACMLResponse(response);
		DecisionType decision = xacmlResponse.getResult().getDecision();
		if (decision.getDecision() == DecisionType.DECISION.Permit){
			this.authorized = true;
		}
		else if (decision.getDecision() == DecisionType.DECISION.Deny){
			this.authorized = false;
		}
		//decision = indeterminate or not applicable
		else {
			
		}
		StatusType status = xacmlResponse.getResult().getStatus();
		this.statusCode = status.getStatusCode().getValue();

		//paolo-note: the status message is optional, if not specifies the following throws a nullpointerexception
		//this.statusMessage = status.getStatusMessage().getValue();
		if (status.getStatusMessage() != null) {
			this.statusMessage = status.getStatusMessage().getValue();			
		} else {
			this.statusMessage = null;
		}
		
		ObligationsType obligations = xacmlResponse.getResult().getObligations();
		this.consentRequired = false;
		this.notificationRequired = false;
	   	if (obligations != null){
	   		Iterator itr = obligations.getObligations().iterator(); 
	   		while(itr.hasNext()) {	   		
	   			ObligationType ob = (ObligationType) itr.next();
	   			String obligation_id = ob.getObligationId();
	   			if (obligation_id.equals(AttributeReleasePolicy.ATTRRELEASE_OBLIGATION_ID_CONSENT)) this.consentRequired = true;
	   			if (obligation_id.equals(AttributeReleasePolicy.ATTRRELEASE_OBLIGATION_ID_NOTIFICATION)) this.notificationRequired = true;	   				    	   		
	   		}
	   	}
					
	}
	
	public String getStatusCode() {
		return statusCode;
	}

	public void setStatusCode(String statusCode) {
		this.statusCode = statusCode;
	}
	
	public boolean getConsentRequired() {
		return consentRequired;
	}

	public void setConsentRequired(boolean consentRequired) {
		this.consentRequired = consentRequired;
	}
	
	public boolean getNotificationRequired() {
		return notificationRequired;
	}

	public void setNotificationRequired(boolean notificationRequired) {
		this.notificationRequired = notificationRequired;
	}
	
	public String getStatusMessage() {
		return statusMessage;
	}

	public void setStatusMessage(String statusMessage) {
		this.statusMessage = statusMessage;
	}
	
	@Override
	public boolean isAuthorized() {
		return authorized;
	}

	public void setAuthorized(boolean authorized) {
		this.authorized = authorized;
	}
	
}
