package eu.semiramis.policymodel.response;

import javax.xml.parsers.ParserConfigurationException;

import org.opensaml.xacml.ctx.DecisionType;
import org.opensaml.xacml.ctx.ResponseType;
import org.opensaml.xacml.ctx.StatusType;
import org.opensaml.xml.ConfigurationException;

import eu.semiramis.policymodel.utils.PolicyModelXACMLResponseUtils;

//paolo-note: the code in this class does not depends on the request type,
//you should create a DefaultResponseHandler class to put this code in common for all the response types and extend it just if needed
//(e.g to extract obligations that are specific of a given response type)
public class RoleBasedAccessResponseHandler extends PolicyModelXACMLResponseUtils implements ResponseHandler{

	private boolean authorized;
	private String statusCode;
	private String statusMessage;
	
	@Override
	public void handle(ResponseType response) {
		try {
			this.init();
		} catch (ConfigurationException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
		ResponseType xacmlResponse = unMarshallXACMLResponse(response);
		DecisionType decision = xacmlResponse.getResult().getDecision();
		if (decision.getDecision() == DecisionType.DECISION.Permit){
			this.authorized = true;
		}
		//paolo-note: I would say that if the decision is not permit then the access should be denied in every case 
		//(including "indeterminate" or "not applicable" decisions)
		else if (decision.getDecision() == DecisionType.DECISION.Deny){
			this.authorized = false;
		}
		//decision = indeterminate or not applicable
		else {
			
		}
		StatusType status = xacmlResponse.getResult().getStatus();
		this.statusCode = status.getStatusCode().getValue();
		
		//paolo-note: the status message is optional, if not specifies the following throws a nullpointerexception
		//this.statusMessage = status.getStatusMessage().getValue();
		if (status.getStatusMessage() != null) {
			this.statusMessage = status.getStatusMessage().getValue();			
		} else {
			this.statusMessage = null;
		}

			
		
	}
	
	public String getStatusCode() {
		return statusCode;
	}

	public void setStatusCode(String statusCode) {
		this.statusCode = statusCode;
	}
	
	public String getStatusMessage() {
		return statusMessage;
	}

	public void setStatusMessage(String statusMessage) {
		this.statusMessage = statusMessage;
	}
	
	@Override
	public boolean isAuthorized() {
		return authorized;
	}

	public void setAuthorized(boolean authorized) {
		this.authorized = authorized;
	}
	
}
