package eu.semiramis.policymodel.utils;

import java.io.InputStream;

import org.opensaml.xacml.XACMLConstants;

import org.opensaml.xacml.policy.ActionType;
import org.opensaml.xacml.policy.ActionMatchType;
import org.opensaml.xacml.policy.ActionsType;
import org.opensaml.xacml.policy.ApplyType;
import org.opensaml.xacml.policy.AttributeDesignatorType;
import org.opensaml.xacml.policy.AttributeValueType;
import org.opensaml.xacml.policy.ConditionType;
import org.opensaml.xacml.policy.DescriptionType;
import org.opensaml.xacml.policy.EffectType;
import org.opensaml.xacml.policy.ExpressionType;
import org.opensaml.xacml.policy.ObligationType;
import org.opensaml.xacml.policy.ObligationsType;
import org.opensaml.xacml.policy.PolicyType;
import org.opensaml.xacml.policy.ResourceMatchType;
import org.opensaml.xacml.policy.ResourceType;
import org.opensaml.xacml.policy.ResourcesType;
import org.opensaml.xacml.policy.RuleType;
import org.opensaml.xacml.policy.SubjectMatchType;
import org.opensaml.xacml.policy.SubjectType;
import org.opensaml.xacml.policy.SubjectsType;
import org.opensaml.xacml.policy.TargetType;
import org.opensaml.xacml.policy.impl.*;
import org.opensaml.xml.io.Marshaller;
import org.opensaml.xml.io.MarshallingException;
import org.opensaml.xml.io.Unmarshaller;
import org.opensaml.xml.io.UnmarshallingException;
import org.opensaml.xml.parse.BasicParserPool;
import org.opensaml.xml.util.XMLHelper;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import eu.semiramis.policyManager.util.opensaml.XACMLUtils;
import eu.semiramis.policymodel.policy.AttributeReleasePolicy;
import eu.semiramis.policymodel.policy.Policy;
import eu.semiramis.policymodel.policy.RoleBasedAccessPolicy;
import eu.semiramis.policymodel.policy.ServiceToServicePolicy;

public class PolicyModelXACMLPolicyUtils extends PolicyModelXACMLUtils {

	protected static String PERMIT_OVERRIDES="urn:oasis:names:tc:xacml:1.0:rule-combining-algorithm:permit-overrides";
	protected static String DENY_OVERRIDES="urn:oasis:names:tc:xacml:1.0:rule-combining-algorithm:deny-overrides";
	enum combiningAlg {permit, deny}; //others can be added if necessary
	
	//builds XACML Policy from Policy Model policy, called at "toXACMLPolicy()" method
	protected PolicyType buildXACMLPolicy(Policy p) {
		
		PolicyType policy = XACMLUtils.create(PolicyType.class, PolicyType.DEFAULT_ELEMENT_NAME);
		//Version attribute for policy should indicate version about the policy itself, not about XACML
		//policy.setVersion(XACMLConstants.XACML20_NS);
		policy.setPolicyId(p.getPolicyId());
		DescriptionType desc = XACMLUtils.create(DescriptionType.class, DescriptionType.DEFAULT_ELEMENT_NAME);
		desc.setValue(p.getDescription());
		policy.setDescription(desc);
							
		if(p instanceof ServiceToServicePolicy) {
			return buildServ2ServXACMLPolicy(p, policy);
		} else if(p instanceof AttributeReleasePolicy) {
			return buildAttrReleaseXACMLPolicy_v2((AttributeReleasePolicy) p, policy);		
		} else if(p instanceof RoleBasedAccessPolicy) {
			return buildRoleBasedXACMLPolicy(p, policy);				
		} else {
			//TODO wrong type exception
		}
		return null;		
	}
	
	//finishes building AttributeRelease XACML Policy from Policy Model policy
	//The resource is included directly in the Condition element, with the different values separated by ";". As such,
	//it does not support wildcards
	protected PolicyType buildAttrReleaseXACMLPolicy_v1(AttributeReleasePolicy p, PolicyType policy) {
		
		combiningAlg alg = combiningAlg.valueOf(p.getCombiningAlgorithm());
		switch (alg) {
			case permit: policy.setRuleCombiningAlgoId(PERMIT_OVERRIDES); break;
			case deny: policy.setRuleCombiningAlgoId(DENY_OVERRIDES); break;
		}
		
		//-------------POLICY TARGET----------------//
		TargetType target = XACMLUtils.create(TargetType.class, TargetType.DEFAULT_ELEMENT_NAME);
		//subject attribute designator
		AttributeDesignatorType subAttDesignator = XACMLUtils.create(AttributeDesignatorType.class, AttributeDesignatorType.SUBJECT_ATTRIBUTE_DESIGNATOR_ELEMENT_NAME);
		subAttDesignator.setDataType("http://www.w3.org/2001/XMLSchema#string");
		subAttDesignator.setAttribtueId(AttributeReleasePolicy.ATTRRELEASE_SUBJECT);
		//subject attribute value
		AttributeValueType subMatchValue = XACMLUtils.create(AttributeValueType.class, AttributeValueType.DEFAULT_ELEMENT_NAME);
		subMatchValue.setDataType("http://www.w3.org/2001/XMLSchema#string");
		subMatchValue.setValue(((AttributeReleasePolicy)p).getRequester());
		//subject match
		SubjectMatchType subjectMatch = XACMLUtils.create(SubjectMatchType.class, SubjectMatchType.DEFAULT_ELEMENT_NAME);
		//maybe "...:anyURI-equal" below
		subjectMatch.setMatchId("urn:oasis:names:tc:xacml:1.0:function:string-equal");
		subjectMatch.setAttributeValue(subMatchValue);					
		subjectMatch.setSubjectAttributeDesignator(subAttDesignator);					
		//subject
		SubjectType subject = XACMLUtils.create(SubjectType.class, SubjectType.DEFAULT_ELEMENT_NAME);
		subject.getSubjectMatches().add(subjectMatch);
		//resources
		SubjectsType subjects = XACMLUtils.create(SubjectsType.class, SubjectsType.DEFAULT_ELEMENT_NAME);
		subjects.getSubjects().add(subject);
		//empty subject
		ActionsType actions = XACMLUtils.create(ActionsType.class, ActionsType.DEFAULT_ELEMENT_NAME);
		//Add to the target
		target.setSubjects(subjects);
		//target.setResources(resources);
		//target.setActions(actions);
				
		policy.setTarget(target);
		
		//-------------POLICY RULE----------------//
		RuleType rule = XACMLUtils.create(RuleType.class, RuleType.DEFAULT_ELEMENT_NAME);
		rule.setRuleId("Attribute Release rule 1");
		switch (alg) {
		case permit: rule.setEffect(EffectType.Permit); break;
		case deny: rule.setEffect(EffectType.Deny); break;
	}
		
		//Rule target
		
		//Add condition
		AttributeValueType attrValue = XACMLUtils.create(AttributeValueType.class, AttributeValueType.DEFAULT_ELEMENT_NAME);
		ConditionType condition = XACMLUtils.create(ConditionType.class, ConditionType.DEFAULT_ELEMENT_NAME);
		ExpressionType expression = XACMLUtils.create(ConditionType.class, ConditionType.DEFAULT_ELEMENT_NAME);
		ApplyType apply = XACMLUtils.create(ApplyType.class, ApplyType.DEFAULT_ELEMENT_NAME);
		ApplyType applyWrapResAttrDesignator = XACMLUtils.create(ApplyType.class, ApplyType.DEFAULT_ELEMENT_NAME);
		AttributeDesignatorType resAttDesignator = XACMLUtils.create(AttributeDesignatorType.class, AttributeDesignatorType.RESOURCE_ATTRIBUTE_DESIGNATOR_ELEMENT_NAME);		
		resAttDesignator.setDataType("http://www.w3.org/2001/XMLSchema#string");
		resAttDesignator.setAttribtueId(AttributeReleasePolicy.ATTRRELEASE_RESOURCE_ATTRIBUTE_ID);
		//resource attribute value
		AttributeValueType resCondValue= XACMLUtils.create(AttributeValueType.class, AttributeValueType.DEFAULT_ELEMENT_NAME);
		resCondValue.setDataType("http://www.w3.org/2001/XMLSchema#string");
		String combinedResource = ((AttributeReleasePolicy)p).getAttrProvider() + ";" + ((AttributeReleasePolicy)p).getAttributeOwner()
			+ ";" + ((AttributeReleasePolicy)p).getAttributeName();
		resCondValue.setValue(combinedResource);
		apply.setFunctionId("urn:oasis:names:tc:xacml:1.0:function:string-equal");
		applyWrapResAttrDesignator.setFunctionId("urn:oasis:names:tc:xacml:1.0:function:string-one-and-only");
		applyWrapResAttrDesignator.getExpressions().add(resAttDesignator);
		apply.getExpressions().add(resCondValue);
		apply.getExpressions().add(applyWrapResAttrDesignator);
		//apply.set
		condition.setExpression(apply);
		//setAttributeValue(subCondValue);
		rule.setCondition(condition);
			
		//Add rule
		policy.getRules().add(rule);
		
		//Add Obligations
		if (((AttributeReleasePolicy)p).getConsentRequired() || ((AttributeReleasePolicy)p).getNotificationRequired()){
			ObligationsType obligations = XACMLUtils.create(ObligationsType.class, ObligationsType.DEFAULT_ELEMENT_QNAME);
			ObligationType obligationConsent = XACMLUtils.create(ObligationType.class, ObligationType.DEFAULT_ELEMENT_QNAME);
			obligationConsent.setObligationId(AttributeReleasePolicy.ATTRRELEASE_OBLIGATION_ID_CONSENT);
			obligationConsent.setFulfillOn(EffectType.Permit);
			ObligationType obligationNotify = XACMLUtils.create(ObligationType.class, ObligationType.DEFAULT_ELEMENT_QNAME);
			obligationNotify.setObligationId(AttributeReleasePolicy.ATTRRELEASE_OBLIGATION_ID_NOTIFICATION);
			obligationNotify.setFulfillOn(EffectType.Permit);
			if (((AttributeReleasePolicy)p).getConsentRequired()) obligations.getObligations().add(obligationConsent);
			if (((AttributeReleasePolicy)p).getNotificationRequired()) obligations.getObligations().add(obligationNotify);
			policy.setObligations(obligations);
		}	
		return policy;		
	}
	
	//finishes building AttributeRelease XACML Policy from Policy Model policy
	//The resources are included in the "target" part of the policy, not in the Condition. As such,
	//it supports wildcards
	protected PolicyType buildAttrReleaseXACMLPolicy_v2(AttributeReleasePolicy p, PolicyType policy) {
		
		combiningAlg alg = combiningAlg.valueOf(p.getCombiningAlgorithm());
		switch (alg) {
			case permit: policy.setRuleCombiningAlgoId(PERMIT_OVERRIDES); break;
			case deny: policy.setRuleCombiningAlgoId(DENY_OVERRIDES); break;
		}
		
		//-------------POLICY TARGET----------------//
		TargetType target = XACMLUtils.create(TargetType.class, TargetType.DEFAULT_ELEMENT_NAME);
		
		if (!((AttributeReleasePolicy)p).getRequester().equals("")) {
			//subject attribute designator
			AttributeDesignatorType subAttDesignator = XACMLUtils.create(AttributeDesignatorType.class, AttributeDesignatorType.SUBJECT_ATTRIBUTE_DESIGNATOR_ELEMENT_NAME);
			subAttDesignator.setDataType("http://www.w3.org/2001/XMLSchema#string");
			subAttDesignator.setAttribtueId(AttributeReleasePolicy.ATTRRELEASE_SUBJECT);
			//subject attribute value
			AttributeValueType subMatchValue = XACMLUtils.create(AttributeValueType.class, AttributeValueType.DEFAULT_ELEMENT_NAME);
			subMatchValue.setDataType("http://www.w3.org/2001/XMLSchema#string");
			subMatchValue.setValue(((AttributeReleasePolicy)p).getRequester());
			//subject match
			SubjectMatchType subjectMatch = XACMLUtils.create(SubjectMatchType.class, SubjectMatchType.DEFAULT_ELEMENT_NAME);
			//maybe "...:anyURI-equal" below
			subjectMatch.setMatchId("urn:oasis:names:tc:xacml:1.0:function:string-equal");
			subjectMatch.setAttributeValue(subMatchValue);					
			subjectMatch.setSubjectAttributeDesignator(subAttDesignator);					
			//subject
			SubjectType subject = XACMLUtils.create(SubjectType.class, SubjectType.DEFAULT_ELEMENT_NAME);
			subject.getSubjectMatches().add(subjectMatch);
			//subjects
			SubjectsType subjects = XACMLUtils.create(SubjectsType.class, SubjectsType.DEFAULT_ELEMENT_NAME);
			subjects.getSubjects().add(subject);
			//Add to the target
			target.setSubjects(subjects);
		} //else empty subject
		
		ActionsType actions = XACMLUtils.create(ActionsType.class, ActionsType.DEFAULT_ELEMENT_NAME);
		
		//resources attribute designator
		AttributeDesignatorType attrProviderAttDesignator = XACMLUtils.create(AttributeDesignatorType.class, AttributeDesignatorType.RESOURCE_ATTRIBUTE_DESIGNATOR_ELEMENT_NAME);
		attrProviderAttDesignator.setDataType("http://www.w3.org/2001/XMLSchema#string");
		attrProviderAttDesignator.setAttribtueId(AttributeReleasePolicy.ATTRRELEASE_RESOURCE_ATTRIBUTE_PROVIDER);
		
		AttributeDesignatorType attrOwnerAttDesignator = XACMLUtils.create(AttributeDesignatorType.class, AttributeDesignatorType.RESOURCE_ATTRIBUTE_DESIGNATOR_ELEMENT_NAME);
		attrOwnerAttDesignator.setDataType("http://www.w3.org/2001/XMLSchema#string");
		attrOwnerAttDesignator.setAttribtueId(AttributeReleasePolicy.ATTRRELEASE_RESOURCE_ATTRIBUTE_OWNER);
		
		AttributeDesignatorType attrNameAttDesignator = XACMLUtils.create(AttributeDesignatorType.class, AttributeDesignatorType.RESOURCE_ATTRIBUTE_DESIGNATOR_ELEMENT_NAME);
		attrNameAttDesignator.setDataType("http://www.w3.org/2001/XMLSchema#string");
		attrNameAttDesignator.setAttribtueId(AttributeReleasePolicy.ATTRRELEASE_RESOURCE_ATTRIBUTE_NAME);
		//resources attribute value
		AttributeValueType attrProviderMatchValue = XACMLUtils.create(AttributeValueType.class, AttributeValueType.DEFAULT_ELEMENT_NAME);
		attrProviderMatchValue.setDataType("http://www.w3.org/2001/XMLSchema#string");
		attrProviderMatchValue.setValue(((AttributeReleasePolicy)p).getAttrProvider());
		
		AttributeValueType attrOwnerMatchValue = XACMLUtils.create(AttributeValueType.class, AttributeValueType.DEFAULT_ELEMENT_NAME);
		attrOwnerMatchValue.setDataType("http://www.w3.org/2001/XMLSchema#string");
		attrOwnerMatchValue.setValue(((AttributeReleasePolicy)p).getAttributeOwner());
		
		AttributeValueType attrNameMatchValue = XACMLUtils.create(AttributeValueType.class, AttributeValueType.DEFAULT_ELEMENT_NAME);
		attrNameMatchValue.setDataType("http://www.w3.org/2001/XMLSchema#string");
		attrNameMatchValue.setValue(((AttributeReleasePolicy)p).getAttributeName());
		
		//resource match
		ResourceMatchType attrProviderMatch = XACMLUtils.create(ResourceMatchType.class, ResourceMatchType.DEFAULT_ELEMENT_NAME);
		attrProviderMatch.setMatchId("urn:oasis:names:tc:xacml:1.0:function:string-equal");
		attrProviderMatch.setAttributeValue(attrProviderMatchValue);					
		attrProviderMatch.setResourceAttributeDesignator(attrProviderAttDesignator);
		
		ResourceMatchType attrOwnerMatch = XACMLUtils.create(ResourceMatchType.class, ResourceMatchType.DEFAULT_ELEMENT_NAME);
		attrOwnerMatch.setMatchId("urn:oasis:names:tc:xacml:1.0:function:string-equal");
		attrOwnerMatch.setAttributeValue(attrOwnerMatchValue);					
		attrOwnerMatch.setResourceAttributeDesignator(attrOwnerAttDesignator);
		
		ResourceMatchType attrNameMatch = XACMLUtils.create(ResourceMatchType.class, ResourceMatchType.DEFAULT_ELEMENT_NAME);
		attrNameMatch.setMatchId("urn:oasis:names:tc:xacml:1.0:function:string-equal");
		attrNameMatch.setAttributeValue(attrNameMatchValue);					
		attrNameMatch.setResourceAttributeDesignator(attrNameAttDesignator);
		//resource
		ResourceType attrResource = XACMLUtils.create(ResourceType.class, ResourceType.DEFAULT_ELEMENT_NAME);
		if (!((AttributeReleasePolicy)p).getAttrProvider().equals("")) {attrResource.getResourceMatches().add(attrProviderMatch);}
		if (!((AttributeReleasePolicy)p).getAttributeOwner().equals("")) {attrResource.getResourceMatches().add(attrOwnerMatch);}
		if (!((AttributeReleasePolicy)p).getAttributeName().equals("")) {attrResource.getResourceMatches().add(attrNameMatch);}
		
		//resources
		ResourcesType resources = XACMLUtils.create(ResourcesType.class, ResourcesType.DEFAULT_ELEMENT_NAME);
		resources.getResources().add(attrResource);
		
		//add to the target
		target.setResources(resources);
		//target.setActions(actions);
				
		policy.setTarget(target);
		
		//-------------POLICY RULE----------------//
		RuleType rule = XACMLUtils.create(RuleType.class, RuleType.DEFAULT_ELEMENT_NAME);
		rule.setRuleId("Attribute Release rule 1");
		switch (alg) {
		case permit: rule.setEffect(EffectType.Permit); break;
		case deny: rule.setEffect(EffectType.Deny); break;
	}
		
		//Rule target
		
		//Add condition
		//ConditionType condition = XACMLUtils.create(ConditionType.class, ConditionType.DEFAULT_ELEMENT_NAME);
		//rule.setCondition(condition);
			
		//Add rule
		policy.getRules().add(rule);
		
		//Add Obligations
		if (((AttributeReleasePolicy)p).getConsentRequired() || ((AttributeReleasePolicy)p).getNotificationRequired()){
			ObligationsType obligations = XACMLUtils.create(ObligationsType.class, ObligationsType.DEFAULT_ELEMENT_QNAME);
			ObligationType obligationConsent = XACMLUtils.create(ObligationType.class, ObligationType.DEFAULT_ELEMENT_QNAME);
			obligationConsent.setObligationId(AttributeReleasePolicy.ATTRRELEASE_OBLIGATION_ID_CONSENT);
			obligationConsent.setFulfillOn(EffectType.Permit);
			ObligationType obligationNotify = XACMLUtils.create(ObligationType.class, ObligationType.DEFAULT_ELEMENT_QNAME);
			obligationNotify.setObligationId(AttributeReleasePolicy.ATTRRELEASE_OBLIGATION_ID_NOTIFICATION);
			obligationNotify.setFulfillOn(EffectType.Permit);
			if (((AttributeReleasePolicy)p).getConsentRequired()) obligations.getObligations().add(obligationConsent);
			if (((AttributeReleasePolicy)p).getNotificationRequired()) obligations.getObligations().add(obligationNotify);
			policy.setObligations(obligations);
		}	
		return policy;		
	}
	
	
	//finishes building ServiceToService XACML Policy from Policy Model policy
	protected PolicyType buildServ2ServXACMLPolicy(Policy p, PolicyType policy) {
		
		policy.setRuleCombiningAlgoId(PERMIT_OVERRIDES);
				
		//paolo-note: don't know why but the builder by default assumes the SAML1 namespace instead of a XACML 2.0, 
		//you can avoid the error by forcing the right object type at creation time.
		//AttributeDesignatorType subAttDesignator = 	subAtrDesBuilder.buildObject(AttributeDesignatorType.SUBJECT_ATTRIBUTE_DESIGNATOR_ELEMENT_NAME);
				
		//-------------POLICY TARGET----------------//
		TargetType target = XACMLUtils.create(TargetType.class, TargetType.DEFAULT_ELEMENT_NAME);
		//empty subject
		SubjectsType subjects = XACMLUtils.create(SubjectsType.class, SubjectsType.DEFAULT_ELEMENT_NAME);
		//resource attribute designator
		AttributeDesignatorType resAttDesignator = XACMLUtils.create(AttributeDesignatorType.class, AttributeDesignatorType.RESOURCE_ATTRIBUTE_DESIGNATOR_ELEMENT_NAME);
		resAttDesignator.setDataType("http://www.w3.org/2001/XMLSchema#string");
		resAttDesignator.setAttribtueId(ServiceToServicePolicy.IATOIA_RESOURCE_ID);
		//resource attribute value
		AttributeValueType resMatchValue = XACMLUtils.create(AttributeValueType.class, AttributeValueType.DEFAULT_ELEMENT_NAME);
		resMatchValue.setDataType("http://www.w3.org/2001/XMLSchema#string");
		resMatchValue.setValue(((ServiceToServicePolicy)p).getTargetService());
		//resource match
		ResourceMatchType resourceMatch = XACMLUtils.create(ResourceMatchType.class, ResourceMatchType.DEFAULT_ELEMENT_NAME);
		//maybe "...:anyURI-equal" below
		resourceMatch.setMatchId("urn:oasis:names:tc:xacml:1.0:function:string-equal");
		resourceMatch.setAttributeValue(resMatchValue);					
		resourceMatch.setResourceAttributeDesignator(resAttDesignator);					
		//resource
		ResourceType resource = XACMLUtils.create(ResourceType.class, ResourceType.DEFAULT_ELEMENT_NAME);
		resource.getResourceMatches().add(resourceMatch);
		//resources
		ResourcesType resources = XACMLUtils.create(ResourcesType.class, ResourcesType.DEFAULT_ELEMENT_NAME);
		resources.getResources().add(resource);
		//empty subject
		ActionsType actions = XACMLUtils.create(ActionsType.class, ActionsType.DEFAULT_ELEMENT_NAME);
		//Add to the target
		//target.setSubjects(subjects);
		target.setResources(resources);
		//target.setActions(actions);
				
		policy.setTarget(target);
		
		//-------------POLICY RULE----------------//
		RuleType rule = XACMLUtils.create(RuleType.class, RuleType.DEFAULT_ELEMENT_NAME);
		rule.setRuleId("ServiceToService rule 1");
		rule.setEffect(EffectType.Permit);
		//Rule target
		TargetType ruleTarget = XACMLUtils.create(TargetType.class, TargetType.DEFAULT_ELEMENT_NAME);
		//empty subjects
		SubjectsType ruleSubjects = XACMLUtils.create(SubjectsType.class, SubjectsType.DEFAULT_ELEMENT_NAME);
		//empty resources
		ResourcesType ruleResources = XACMLUtils.create(ResourcesType.class, ResourcesType.DEFAULT_ELEMENT_NAME);
		//action attribute designator
		AttributeDesignatorType actAttDesignator = XACMLUtils.create(AttributeDesignatorType.class, AttributeDesignatorType.ACTION_ATTRIBUTE_DESIGNATOR_ELEMENT_NAME);
		actAttDesignator.setDataType("http://www.w3.org/2001/XMLSchema#string");
		actAttDesignator.setAttribtueId(ServiceToServicePolicy.IATOIA_ACTION_ID);
		//action attribute value
		AttributeValueType actMatchValue = XACMLUtils.create(AttributeValueType.class, AttributeValueType.DEFAULT_ELEMENT_NAME);
		actMatchValue.setDataType("http://www.w3.org/2001/XMLSchema#string");
		actMatchValue.setValue(((ServiceToServicePolicy)p).getFunctionality());
		//action match
		ActionMatchType actionMatch = XACMLUtils.create(ActionMatchType.class, ActionMatchType.DEFAULT_ELEMENT_NAME);
		//maybe "...:anyURI-equal" below
		actionMatch.setMatchId("urn:oasis:names:tc:xacml:1.0:function:string-equal");
		actionMatch.setAttributeValue(actMatchValue);					
		actionMatch.setActionAttributeDesignator(actAttDesignator);					
		//resource
		ActionType ruleAction = XACMLUtils.create(ActionType.class, ActionType.DEFAULT_ELEMENT_NAME);
		ruleAction.getActionMatches().add(actionMatch);
		//resources
		ActionsType ruleActions = XACMLUtils.create(ActionsType.class, ActionsType.DEFAULT_ELEMENT_NAME);
		ruleActions.getActions().add(ruleAction);
		//Add to the target
		//ruleTarget.setSubjects(ruleSubjects);
		//ruleTarget.setResources(ruleResources);
		ruleTarget.setActions(ruleActions);
		
		rule.setTarget(ruleTarget);
		
		//Add condition
		AttributeValueType attrValue = XACMLUtils.create(AttributeValueType.class, AttributeValueType.DEFAULT_ELEMENT_NAME);
		ConditionType condition = XACMLUtils.create(ConditionType.class, ConditionType.DEFAULT_ELEMENT_NAME);
		ExpressionType expression = XACMLUtils.create(ConditionType.class, ConditionType.DEFAULT_ELEMENT_NAME);
		ApplyType apply = XACMLUtils.create(ApplyType.class, ApplyType.DEFAULT_ELEMENT_NAME);
		ApplyType applyWrapSubAttrDesignator = XACMLUtils.create(ApplyType.class, ApplyType.DEFAULT_ELEMENT_NAME);
		AttributeDesignatorType subAttDesignator = XACMLUtils.create(AttributeDesignatorType.class, AttributeDesignatorType.SUBJECT_ATTRIBUTE_DESIGNATOR_ELEMENT_NAME);		
		subAttDesignator.setDataType("http://www.w3.org/2001/XMLSchema#string");
		//subAttDesignator.setAttribtueId(ServiceToServicePolicy.SUBJECT_ID);
		subAttDesignator.setAttribtueId(ServiceToServicePolicy.IATOIA_SUBJECT_ID);
		//subject attribute value
		AttributeValueType subCondValue= XACMLUtils.create(AttributeValueType.class, AttributeValueType.DEFAULT_ELEMENT_NAME);
		subCondValue.setDataType("http://www.w3.org/2001/XMLSchema#string");
		subCondValue.setValue(((ServiceToServicePolicy)p).getOriginService());
		apply.setFunctionId("urn:oasis:names:tc:xacml:1.0:function:string-equal");
		applyWrapSubAttrDesignator.setFunctionId("urn:oasis:names:tc:xacml:1.0:function:string-one-and-only");
		applyWrapSubAttrDesignator.getExpressions().add(subAttDesignator);
		//applyWrapSubAttrDesignator.getExpressions().add(subCondValue);
		apply.getExpressions().add(applyWrapSubAttrDesignator);
		apply.getExpressions().add(subCondValue);
		//apply.set
		condition.setExpression(apply);
		//setAttributeValue(subCondValue);
		rule.setCondition(condition);
		//Add rule
		policy.getRules().add(rule);
		
		//Obligations if necessary
		//policy.setObligations(obligationstype)
		
		return policy;		
	}
	
	//finishes building RoleBasedAccess XACML Policy from Policy Model policy
	protected PolicyType buildRoleBasedXACMLPolicy(Policy p, PolicyType policy) {
		
		policy.setRuleCombiningAlgoId(PERMIT_OVERRIDES);
		
		//-------------POLICY TARGET----------------//
		TargetType target = XACMLUtils.create(TargetType.class, TargetType.DEFAULT_ELEMENT_NAME);
		//empty subject
		SubjectsType subjects = XACMLUtils.create(SubjectsType.class, SubjectsType.DEFAULT_ELEMENT_NAME);
		//resource attribute designator
		AttributeDesignatorType resAttDesignator = XACMLUtils.create(AttributeDesignatorType.class, AttributeDesignatorType.RESOURCE_ATTRIBUTE_DESIGNATOR_ELEMENT_NAME);
		resAttDesignator.setDataType("http://www.w3.org/2001/XMLSchema#string");
		resAttDesignator.setAttribtueId(RoleBasedAccessPolicy.RBAC_RESOURCE_ID);
		//resource attribute value
		AttributeValueType resMatchValue = XACMLUtils.create(AttributeValueType.class, AttributeValueType.DEFAULT_ELEMENT_NAME);
		resMatchValue.setDataType("http://www.w3.org/2001/XMLSchema#string");
		resMatchValue.setValue(((RoleBasedAccessPolicy)p).getRequestedResource());
		//resource match
		ResourceMatchType resourceMatch = XACMLUtils.create(ResourceMatchType.class, ResourceMatchType.DEFAULT_ELEMENT_NAME);
		//maybe "...:anyURI-equal" below
		resourceMatch.setMatchId("urn:oasis:names:tc:xacml:1.0:function:string-equal");
		resourceMatch.setAttributeValue(resMatchValue);					
		resourceMatch.setResourceAttributeDesignator(resAttDesignator);					
		//resource
		ResourceType resource = XACMLUtils.create(ResourceType.class, ResourceType.DEFAULT_ELEMENT_NAME);
		resource.getResourceMatches().add(resourceMatch);
		//resources
		ResourcesType resources = XACMLUtils.create(ResourcesType.class, ResourcesType.DEFAULT_ELEMENT_NAME);
		resources.getResources().add(resource);
		//empty subject
		ActionsType actions = XACMLUtils.create(ActionsType.class, ActionsType.DEFAULT_ELEMENT_NAME);
		//Add to the target
		//target.setSubjects(subjects);
		target.setResources(resources);
		//target.setActions(actions);
				
		policy.setTarget(target);
		
		//-------------POLICY RULE----------------//
		RuleType rule = XACMLUtils.create(RuleType.class, RuleType.DEFAULT_ELEMENT_NAME);
		rule.setRuleId("RoleBasedAccess rule 1");
		rule.setEffect(EffectType.Permit);
		//Rule target
		TargetType ruleTarget = XACMLUtils.create(TargetType.class, TargetType.DEFAULT_ELEMENT_NAME);
		//empty subjects
		SubjectsType ruleSubjects = XACMLUtils.create(SubjectsType.class, SubjectsType.DEFAULT_ELEMENT_NAME);
		//empty resources
		ResourcesType ruleResources = XACMLUtils.create(ResourcesType.class, ResourcesType.DEFAULT_ELEMENT_NAME);
		//action attribute designator
		AttributeDesignatorType actAttDesignator = XACMLUtils.create(AttributeDesignatorType.class, AttributeDesignatorType.ACTION_ATTRIBUTE_DESIGNATOR_ELEMENT_NAME);
		actAttDesignator.setDataType("http://www.w3.org/2001/XMLSchema#string");
		actAttDesignator.setAttribtueId(RoleBasedAccessPolicy.RBAC_ACTION_ID);
		//action attribute value
		AttributeValueType actMatchValue = XACMLUtils.create(AttributeValueType.class, AttributeValueType.DEFAULT_ELEMENT_NAME);
		actMatchValue.setDataType("http://www.w3.org/2001/XMLSchema#string");
		actMatchValue.setValue(((RoleBasedAccessPolicy)p).getFunctionalityId());
		//action match
		ActionMatchType actionMatch = XACMLUtils.create(ActionMatchType.class, ActionMatchType.DEFAULT_ELEMENT_NAME);
		//maybe "...:anyURI-equal" below
		actionMatch.setMatchId("urn:oasis:names:tc:xacml:1.0:function:string-equal");
		actionMatch.setAttributeValue(actMatchValue);					
		actionMatch.setActionAttributeDesignator(actAttDesignator);					
		//resource
		ActionType ruleAction = XACMLUtils.create(ActionType.class, ActionType.DEFAULT_ELEMENT_NAME);
		ruleAction.getActionMatches().add(actionMatch);
		//resources
		ActionsType ruleActions = XACMLUtils.create(ActionsType.class, ActionsType.DEFAULT_ELEMENT_NAME);
		ruleActions.getActions().add(ruleAction);
		//Add to the target
		//ruleTarget.setSubjects(ruleSubjects);
		//ruleTarget.setResources(ruleResources);
		ruleTarget.setActions(ruleActions);
		
		rule.setTarget(ruleTarget);
		
		//Add condition
		AttributeValueType attrValue = XACMLUtils.create(AttributeValueType.class, AttributeValueType.DEFAULT_ELEMENT_NAME);
		ConditionType condition = XACMLUtils.create(ConditionType.class, ConditionType.DEFAULT_ELEMENT_NAME);
		ExpressionType expression = XACMLUtils.create(ConditionType.class, ConditionType.DEFAULT_ELEMENT_NAME);
		ApplyType apply = XACMLUtils.create(ApplyType.class, ApplyType.DEFAULT_ELEMENT_NAME);
		ApplyType applyWrapSubAttrDesignator = XACMLUtils.create(ApplyType.class, ApplyType.DEFAULT_ELEMENT_NAME);
		AttributeDesignatorType subAttDesignator = XACMLUtils.create(AttributeDesignatorType.class, AttributeDesignatorType.SUBJECT_ATTRIBUTE_DESIGNATOR_ELEMENT_NAME);		
		subAttDesignator.setDataType("http://www.w3.org/2001/XMLSchema#string");
		//subAttDesignator.setAttribtueId(ServiceToServicePolicy.SUBJECT_ID);
		subAttDesignator.setAttribtueId(RoleBasedAccessPolicy.RBAC_SUBJECT_ID);
		//subject attribute value
		AttributeValueType subCondValue= XACMLUtils.create(AttributeValueType.class, AttributeValueType.DEFAULT_ELEMENT_NAME);
		subCondValue.setDataType("http://www.w3.org/2001/XMLSchema#string");
		subCondValue.setValue(((RoleBasedAccessPolicy)p).getRole());
		apply.setFunctionId("urn:oasis:names:tc:xacml:1.0:function:string-equal");
		applyWrapSubAttrDesignator.setFunctionId("urn:oasis:names:tc:xacml:1.0:function:string-one-and-only");
		applyWrapSubAttrDesignator.getExpressions().add(subAttDesignator);
		
		//paolo-note: the following subCondValue must be added to the external apply
		//applyWrapSubAttrDesignator.getExpressions().add(subCondValue);
		apply.getExpressions().add(subCondValue);

		apply.getExpressions().add(applyWrapSubAttrDesignator);
		//apply.set
		condition.setExpression(apply);
		//setAttributeValue(subCondValue);
		rule.setCondition(condition);
		//Add rule
		policy.getRules().add(rule);
		
		//Obligations if necessary
		//policy.setObligations(obligationstype)
		
		return policy;				
	}
	
	//returns XACML PolicyType as a string
	protected String policyTypeToString(PolicyType policy) {
		String output = "";
		Document doc = db.newDocument();
		
	    Marshaller marshallerPolicy = marshallerFactory
		    .getMarshaller(PolicyType.DEFAULT_ELEMENT_NAME);
	    Element element;
		try {
			element = marshallerPolicy.marshall(policy, doc);
			output = XMLHelper.prettyPrintXML(element);
		} catch (MarshallingException e) {
			e.printStackTrace();
		}
	    return output;
	}
	
	protected PolicyType unMarshallXACMLPolicy(PolicyType policy) {
		try {
		    InputStream in = stringToInputStream(policyTypeToString(policy));
		    BasicParserPool pool = new BasicParserPool();
		    Document doc = pool.parse(in);
		    Unmarshaller reqUnmarshaller = unMarshallerFactory
			    .getUnmarshaller(PolicyType.DEFAULT_ELEMENT_NAME);
		   return (PolicyType) reqUnmarshaller.unmarshall(doc.getDocumentElement());
		} catch (UnmarshallingException e) {
			System.out.println(e);
		} catch (Exception e) {
			System.out.println(e);
		}
		
		return null;
	}
	
}
