package eu.semiramis.policymodel.utils;

import java.io.InputStream;
import org.opensaml.xacml.XACMLConstants;
import org.opensaml.xacml.ctx.ActionType;
import org.opensaml.xacml.ctx.AttributeType;
import org.opensaml.xacml.ctx.AttributeValueType;
import org.opensaml.xacml.ctx.EnvironmentType;
import org.opensaml.xacml.ctx.RequestType;
import org.opensaml.xacml.ctx.ResourceType;
import org.opensaml.xacml.ctx.SubjectType;
import org.opensaml.xml.io.Marshaller;
import org.opensaml.xml.io.MarshallingException;
import org.opensaml.xml.io.Unmarshaller;
import org.opensaml.xml.io.UnmarshallingException;
import org.opensaml.xml.parse.BasicParserPool;
import org.opensaml.xml.util.XMLHelper;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import eu.semiramis.policyManager.util.opensaml.XACMLUtils;
import eu.semiramis.policymodel.policy.AttributeReleasePolicy;
import eu.semiramis.policymodel.policy.RoleBasedAccessPolicy;
import eu.semiramis.policymodel.policy.ServiceToServicePolicy;
import eu.semiramis.policymodel.request.AttributeReleaseRequest;
import eu.semiramis.policymodel.request.Request;
import eu.semiramis.policymodel.request.RoleBasedAccessRequest;
import eu.semiramis.policymodel.request.ServiceToServiceRequest;

public class PolicyModelXACMLRequestUtils extends PolicyModelXACMLUtils  {
	
	//returns XACML requestType as a string
	protected String requestTypeToString(RequestType request) {
		String output = "";
		Document doc = db.newDocument();
	    Marshaller marshallerRequest = marshallerFactory
		    .getMarshaller(RequestType.DEFAULT_ELEMENT_NAME);
	    Element element;
		try {
			element = marshallerRequest.marshall(request, doc);
			output = XMLHelper.prettyPrintXML(element);
		} catch (MarshallingException e) {
			e.printStackTrace();
		}
	    return output;
	}	
	
	//builds XACML Request from Policy Model request, called at "toXACMLRequest()" method

	//paolo-note: you can replace this selection method by simply overloading the buildXACMLRequest() method
	//with the three kind of access requests we have
	//e.g.: buildXACMLRequest(RoleBasedAccessRequest request) {...}
	//the java dynamic binding will do the magic by calling the right method among the three.
	//This will also save you from managing the invocation of this method with a wrong type (not managed now).
	protected RequestType buildXACMLRequest(Request r) {
		
		RequestType request = XACMLUtils.create(RequestType.class, RequestType.DEFAULT_ELEMENT_NAME);
		
		//paolo-note: it seems that the environment element is needed, even if empty
		request.setEnvironment(XACMLUtils.create(EnvironmentType.class, EnvironmentType.DEFAULT_ELEMENT_NAME));
							
		if(r instanceof ServiceToServiceRequest) {
			return buildServ2ServXACMLRequest(r, request);
		} else if(r instanceof AttributeReleaseRequest) {
			return buildAttrReleaseXACMLRequest(r, request);		
		} else if(r instanceof RoleBasedAccessRequest) {
			return buildRoleBasedXACMLRequest(r, request);				
		} else {
			//TODO wrong type exception
		}
		
		return null;
		
	}	
	
	//finishes building ServiceToService XACML Request from Policy Model request, called at "toXACMLRequest()" method
	protected RequestType buildServ2ServXACMLRequest(Request r, RequestType request) {
		
		//action
		AttributeValueType actAttributeValue = XACMLUtils.create(AttributeValueType.class, AttributeValueType.DEFAULT_ELEMENT_NAME);
		actAttributeValue.setValue((((ServiceToServiceRequest)r).getFunctionality()));
		ActionType action = XACMLUtils.create(ActionType.class, ActionType.DEFAULT_ELEMENT_NAME);
		AttributeType actAttribute = XACMLUtils.create(AttributeType.class, AttributeType.DEFAULT_ELEMENT_NAME);
		actAttribute.setAttributeID(ServiceToServiceRequest.IATOIA_ACTION_ID);
		actAttribute.setDataType("http://www.w3.org/2001/XMLSchema#string");
		actAttribute.getAttributeValues().add(actAttributeValue);
		action.getAttributes().add(actAttribute);
		
		//resource
		AttributeValueType resAttributeValue = XACMLUtils.create(AttributeValueType.class, AttributeValueType.DEFAULT_ELEMENT_NAME);
		resAttributeValue.setValue((((ServiceToServiceRequest)r).getTargetService()));
		ResourceType resource = XACMLUtils.create(ResourceType.class, ResourceType.DEFAULT_ELEMENT_NAME);
		AttributeType resAttribute = XACMLUtils.create(AttributeType.class, AttributeType.DEFAULT_ELEMENT_NAME);
		resAttribute.setAttributeID(ServiceToServiceRequest.IATOIA_RESOURCE_ID);
		resAttribute.setDataType("http://www.w3.org/2001/XMLSchema#string");
		resAttribute.getAttributeValues().add(resAttributeValue);
		resource.getAttributes().add(resAttribute);		
		
		//subject
		AttributeValueType subAttributeValue = XACMLUtils.create(AttributeValueType.class, AttributeValueType.DEFAULT_ELEMENT_NAME);
		subAttributeValue.setValue((((ServiceToServiceRequest)r).getOriginService()));
		SubjectType subject = XACMLUtils.create(SubjectType.class, SubjectType.DEFAULT_ELEMENT_NAME);
		AttributeType subAttribute = XACMLUtils.create(AttributeType.class, AttributeType.DEFAULT_ELEMENT_NAME);
		subAttribute.setAttributeID(ServiceToServiceRequest.IATOIA_SUBJECT_ID);	
		subAttribute.setDataType("http://www.w3.org/2001/XMLSchema#string");
		subAttribute.getAttributeValues().add(subAttributeValue);
		subject.getAttributes().add(subAttribute);

		request.getResources().add(resource);
		request.getSubjects().add(subject);
		request.setAction(action);
		
		return request;
	}
	
	//finishes building AttributeRelease XACML Request from Policy Model request, called at "toXACMLRequest()" method
	protected RequestType buildAttrReleaseXACMLRequest(Request r, RequestType request) {
		
		//resource
		//including code for adding another attributes to the resource
		AttributeValueType resAttrProviderValue = XACMLUtils.create(AttributeValueType.class, AttributeValueType.DEFAULT_ELEMENT_NAME);
		//The line below is in case we want to have the attrProvider, attrOwner and attrName separated by ";" as part of the same attribute
		//String combinedResource = ((AttributeReleaseRequest)r).getAttrProvider() + ";" + ((AttributeReleaseRequest)r).getAttributeOwner()
			//+ ";" + ((AttributeReleaseRequest)r).getRequestedAttributeName();
		resAttrProviderValue.setValue(((AttributeReleaseRequest)r).getAttrProvider());
		AttributeValueType resAttrOwnerValue = XACMLUtils.create(AttributeValueType.class, AttributeValueType.DEFAULT_ELEMENT_NAME);
		resAttrOwnerValue.setValue(((AttributeReleaseRequest)r).getAttributeOwner());
		AttributeValueType resAttrNameValue = XACMLUtils.create(AttributeValueType.class, AttributeValueType.DEFAULT_ELEMENT_NAME);
		resAttrNameValue.setValue(((AttributeReleaseRequest)r).getRequestedAttributeName());
		
		ResourceType resource = XACMLUtils.create(ResourceType.class, ResourceType.DEFAULT_ELEMENT_NAME);
		AttributeType resAttrProvider = XACMLUtils.create(AttributeType.class, AttributeType.DEFAULT_ELEMENT_NAME);
		resAttrProvider.setAttributeID(AttributeReleaseRequest.ATTRRELEASE_RESOURCE_ATTRIBUTE_PROVIDER);
		resAttrProvider.setDataType("http://www.w3.org/2001/XMLSchema#string");
		AttributeType resAttrOwner = XACMLUtils.create(AttributeType.class, AttributeType.DEFAULT_ELEMENT_NAME);
		resAttrOwner.setAttributeID(AttributeReleaseRequest.ATTRRELEASE_RESOURCE_ATTRIBUTE_OWNER);
		resAttrOwner.setDataType("http://www.w3.org/2001/XMLSchema#string");
		AttributeType resAttrName = XACMLUtils.create(AttributeType.class, AttributeType.DEFAULT_ELEMENT_NAME);
		resAttrName.setAttributeID(AttributeReleaseRequest.ATTRRELEASE_RESOURCE_ATTRIBUTE_NAME);
		resAttrName.setDataType("http://www.w3.org/2001/XMLSchema#string");
		resAttrProvider.getAttributeValues().add(resAttrProviderValue);
		resAttrOwner.getAttributeValues().add(resAttrOwnerValue);
		resAttrName.getAttributeValues().add(resAttrNameValue);
			
		resource.getAttributes().add(resAttrProvider);
		resource.getAttributes().add(resAttrOwner);
		resource.getAttributes().add(resAttrName);
		
		//subject
		AttributeValueType subAttributeValue = XACMLUtils.create(AttributeValueType.class, AttributeValueType.DEFAULT_ELEMENT_NAME);
		subAttributeValue.setValue((((AttributeReleaseRequest)r).getRequester()));
		SubjectType subject = XACMLUtils.create(SubjectType.class, SubjectType.DEFAULT_ELEMENT_NAME);
		AttributeType subAttribute = XACMLUtils.create(AttributeType.class, AttributeType.DEFAULT_ELEMENT_NAME);
		subAttribute.setAttributeID(AttributeReleaseRequest.ATTRRELEASE_SUBJECT);	
		subAttribute.setDataType("http://www.w3.org/2001/XMLSchema#string");
		subAttribute.getAttributeValues().add(subAttributeValue);
		subject.getAttributes().add(subAttribute);

		//empty action, required for XACML internal validation
		ActionType action = XACMLUtils.create(ActionType.class, ActionType.DEFAULT_ELEMENT_NAME);
		
		request.getResources().add(resource);
		request.getSubjects().add(subject);
		request.setAction(action);
		
		return request;
	}
	
	//finishes building RolebasedAccess XACML Request from Policy Model request, called at "toXACMLRequest()" method
	protected RequestType buildRoleBasedXACMLRequest(Request r, RequestType request) {
		
		//action
		AttributeValueType actAttributeValue = XACMLUtils.create(AttributeValueType.class, AttributeValueType.DEFAULT_ELEMENT_NAME);
		actAttributeValue.setValue((((RoleBasedAccessRequest)r).getFunctionality()));
		ActionType action = XACMLUtils.create(ActionType.class, ActionType.DEFAULT_ELEMENT_NAME);
		AttributeType actAttribute = XACMLUtils.create(AttributeType.class, AttributeType.DEFAULT_ELEMENT_NAME);
		actAttribute.setAttributeID(RoleBasedAccessRequest.RBAC_ACTION_ID);
		actAttribute.setDataType("http://www.w3.org/2001/XMLSchema#string");
		actAttribute.getAttributeValues().add(actAttributeValue);
		action.getAttributes().add(actAttribute);
		
		//resource
		AttributeValueType resAttributeValue = XACMLUtils.create(AttributeValueType.class, AttributeValueType.DEFAULT_ELEMENT_NAME);
		resAttributeValue.setValue((((RoleBasedAccessRequest)r).getRequestedResource()));
		ResourceType resource = XACMLUtils.create(ResourceType.class, ResourceType.DEFAULT_ELEMENT_NAME);
		AttributeType resAttribute = XACMLUtils.create(AttributeType.class, AttributeType.DEFAULT_ELEMENT_NAME);
		resAttribute.setAttributeID(RoleBasedAccessRequest.RBAC_RESOURCE_ID);
		resAttribute.setDataType("http://www.w3.org/2001/XMLSchema#string");
		resAttribute.getAttributeValues().add(resAttributeValue);
		resource.getAttributes().add(resAttribute);		
		
		//subject
		AttributeValueType subAttributeValue = XACMLUtils.create(AttributeValueType.class, AttributeValueType.DEFAULT_ELEMENT_NAME);
		subAttributeValue.setValue((((RoleBasedAccessRequest)r).getRole()));
		SubjectType subject = XACMLUtils.create(SubjectType.class, SubjectType.DEFAULT_ELEMENT_NAME);
		AttributeType subAttribute = XACMLUtils.create(AttributeType.class, AttributeType.DEFAULT_ELEMENT_NAME);
		subAttribute.setAttributeID(RoleBasedAccessRequest.RBAC_SUBJECT_ID);	
		subAttribute.setDataType("http://www.w3.org/2001/XMLSchema#string");
		subAttribute.getAttributeValues().add(subAttributeValue);
		subject.getAttributes().add(subAttribute);

		request.getResources().add(resource);
		request.getSubjects().add(subject);
		request.setAction(action);
		
		return request;
	}
	
	protected RequestType unMarshallXACMLRequest(RequestType request) {
		try {
		    InputStream in = stringToInputStream(requestTypeToString(request));
		    BasicParserPool pool = new BasicParserPool();
		    Document doc = pool.parse(in);
		    Unmarshaller reqUnmarshaller = unMarshallerFactory
			    .getUnmarshaller(RequestType.DEFAULT_ELEMENT_NAME);
		   return (RequestType) reqUnmarshaller.unmarshall(doc.getDocumentElement());
		} catch (UnmarshallingException e) {
			System.out.println(e);
		} catch (Exception e) {
			System.out.println(e);
		}
		
		return null;
	}
}
